(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.XTouch = factory());
}(this, (function () { 'use strict';

  function tap(status) {
    if (status.diff_Abs_X < status.tapMin && status.diff_Abs_Y < status.tapMin && status.time <= status.longTapTime) {
      status.callback.call(this, status);
    }
  }
  function longTap(status) {
    if (status.diff_Abs_X < status.tapMin && status.diff_Abs_Y < status.tapMin && status.time > status.longTapTime) {
      status.callback.call(this, status);
    }
  }
  function swipUp(status) {
    if (status.diff_Abs_X < status.diff_Abs_Y && status.diff_Abs_Y > status.tapMin && status.diff_Y >= status.slideMin) {
      status.callback.call(this, status);
    }
  }
  function swipDown(status) {
    if (status.diff_Abs_X < status.diff_Abs_Y && status.diff_Abs_Y > status.tapMin && status.diff_Y <= -status.slideMin) {
      status.callback.call(this, status);
    }
  }
  function swipLeft(status) {
    if (status.diff_Abs_X > status.diff_Abs_Y && status.diff_Abs_X > status.tapMin && status.diff_X >= status.slideMin) {
      status.callback.call(this, status);
    }
  }
  function swipRight(status) {
    if (status.diff_Abs_X > status.diff_Abs_Y && status.diff_Abs_X > status.tapMin && status.diff_X <= -status.slideMin) {
      status.callback.call(this, status);
    }
  }

  function addEvent(status) {
    var _this = this;

    this.addEventListener('touchstart', function (e) {
      e.preventDefault();
      status.startScreenX = e.touches[0].screenX;
      status.startScreenY = e.touches[0].screenY;
      status.endScreenX = e.touches[0].screenX;
      status.endScreenY = e.touches[0].screenY;
      status.time = 0;
      status.timer = setInterval(function () {
        status.time = status.time + 10;
      }, 10);
    }, { passive: false });

    this.addEventListener('touchmove', function (e) {
      e.preventDefault();
      status.endScreenX = e.touches[0].screenX;
      status.endScreenY = e.touches[0].screenY;
    }, { passive: false });

    this.addEventListener('touchend', function (e) {
      e.preventDefault();
      clearInterval(status.timer);
      status.diff_X = status.startScreenX - status.endScreenX;
      status.diff_Y = status.startScreenY - status.endScreenY;
      status.diff_Abs_X = Math.abs(status.diff_X);
      status.diff_Abs_Y = Math.abs(status.diff_Y);
      switch (status.type) {
        case 'longTap':
          longTap.call(_this, status);
          break;
        case 'swipUp':
          swipUp.call(_this, status);
          break;
        case 'swipDown':
          swipDown.call(_this, status);
          break;
        case 'swipRight':
          swipRight.call(_this, status);
          break;
        case 'swipLeft':
          swipLeft.call(_this, status);
          break;
        default:
          tap.call(_this, status);
          break;
      }
    }, { passive: false });
  }

  var classCallCheck = function (instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  };

  var createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var XTouch = function () {
    function XTouch(config) {
      classCallCheck(this, XTouch);

      var _config = config || {};
      // 滑动最小距离
      this.slideMin = _config.slideMin || 50;
      // 点击最大距离
      this.tapMin = _config.tapMin || 10;
      // 长按最短时间
      this.longTapTime = _config.longTapTime || 300;
    }

    createClass(XTouch, [{
      key: 'init',
      value: function init() {
        var that = this;
        // 使用function 定义，使this指向事件绑定的DOM
        HTMLElement.prototype.xtouch = function (type, callback) {
          // 初始化状态
          var status = {
            type: type, callback: callback,
            // 触摸起始坐标
            startScreenX: 0,
            startScreenY: 0,
            // 触摸结束坐标
            endScreenX: 0,
            endScreenY: 0,
            // X轴、Y轴坐标差值
            diff_X: 0,
            diff_Y: 0,
            // X轴、Y轴坐标差值绝对值
            diff_Abs_X: 0,
            diff_Abs_Y: 0,
            // 防误触距离
            slideMin: that.slideMin,
            tapMin: that.tapMin,
            longTapTime: that.longTapTime,
            // 计时器
            timer: null,
            time: 0
          };
          addEvent.call(this, status);
        };
      }
    }]);
    return XTouch;
  }();

  return XTouch;

})));
